;Serial Communications Sample 3 (Interrupt-Driven Serial Driver with Reception Buffer)
;Demonstrates the usage of a serial communications driver with a 16-byte reception buffer
;Displays the received data values
;Sends simple data on a regular cycle

* ------------------------------------------------------
*  Special Function Registers

       .include "sfr.i"

* ------------------------------------------------------
*  Variable-Definitions

;Oscillation Control Register settings
;Internal RC oscillation for system clock
osc_rc EQU 77
;Crystal oscillation for system clock
osc_xt EQU 239

;Low battery detection flag (RAM bank 0)
LowBattChk EQU 110

;Serial communications buffer size
SioRxCueSize EQU 16

;For Serial Driver

;Amount of received data waiting
SioRxCueBehind = $30
;Reception buffer reading point
SioRxCueRPnt = $31
;Reception buffer writing point
SioRxCueWPnt = $32
;Reception buffer
SioRxCue = $33
;Reception overrun flag
SioOverRun = $34

;Work Areas for Usage Sample

;Base clock counter
bcount = $35
;Work 1
work1 = $36
;Work 2
work2 = $37

;Work (put2digit)
work0 = $38

* ------------------------------------------------------
*  Interupt Vectors

	.include "libraries/s_intvec.i"

* ------------------------------------------------------
*  Game Data

	.include "data/gamehdr.i"
	.include "data/iconanm.i"
	.include "data/iconplt.i"
	.include "data/icondat.i"

* ------------------------------------------------------
*  Main Program Loop

	.include "libraries/main.i"

* ------------------------------------------------------
*  Jump To User Program

	jmp begin

* ------------------------------------------------------
*  User Defined Imports


* ------------------------------------------------------
*  User Defined Program

;Serial communications driver usage sample
;Sends simple data at a regular interval
;Displays the received data values on the LCD
begin:
	mov #0, bcount
	;Initial value of transmission data
	mov #0, work1
	;Masks P3 interrupts
	clr1 P3INT, 0
	;Clears the LCD
	call cls
	;Turns off the low battery automatic detection function
	call BattChkOff
	;Serial communications initialization
	call SioInit
	;Ends if VM is not connected
	bnz .finish
.finish:
	call finish

stlp1:
	;Displaying if data has been received
	;1-byte reception
	call SioGet1
	;Skip if no data has been received
	be #$ff, stnx1
	;If normal received data is found, go to stnx3
	bz stnx3

;Forcibly terminate if an error is detected
error:
	br .finish
.finish:
	call finish

stnx3:
	;Load received data from b -> acc
	ld b
	;Display coordinate (horizontal)
	mov #0, c
	;Display coordinate (vertical)
	mov #0, b
	;Displays numeric value on the LCD
	call put2digit
	;Continues to repeat as long as there is received data
	br stlp1

stnx1:
	;Waits until next interrupt
	set1 PCON, 0
	
	;Sending simple data at a regular interval
	;Base timer counter value
	ld bcount
	;Does not send if unchanged
	be work2, stnx4
	;Updates work2
	st work2
	;Loads the transmission data
	ld work1
	;Sends
	call SioPut1
	;Updates the transmission data
	inc work1
	;(Sends values form 0 to 99, in sequence)
	ld work1
	bne #100, stnx2
	mov #0, work1

stnx2:
stnx4:
	;[M] (mode) Button Check
	ld P3
	;If the [M] button is pressed, the application ends
	bn acc, 6, .finish
	;Repeat
	jmp stlp1
.finish
	call finish

;End program
finish:
	;Serial communications end processing
	call SioEnd
	;Turns on the low battery automatic detection function
	jmpf goodbye

;Simple Serial Communications Driver

;Serial communications initialization
;This sample assumes that the system clock is in crystal mode.

;VM connection check
SioInit:
	;Checks the connection status
	ld P7
	;Checks P70, P72, P73
	and #%00001101
	;P70 = 0, P72 = 0, P73 = 1
	;To next3 if connected
	be #%00001000, next3
	;If not connected, abnormal end with acc = 0ffh
	mov #$ff, acc
	;SioInit end
	ret

;Serial communications initialization
next3:
	;Specifies output as 'LSB first'
	mov #0, SCON0
	;Specifies input as 'LSB first'
	mov #0, SCON1
	;Sets the transfer rate
	mov #$dd, SBR
	;Clears the P10 latch (P10/S00)
	clr1 P1, 0
	;Clears the P12 latch (P12/SCK0)
	clr1 P1, 2
	;Clears the P13 latch (P13/S01)
	clr1 P1, 3
	;Sets the pin functions
	mov #%00000101, P1FCR
	;Sets the pin functions
	mov #%00000101, P1DDR
	;Clears the transfer buffer
	mov #0, SBUF0
	;Clears the transfer buffer
	mov #0, SBUF1
	mov #0, acc
	;Resets amount of received data waiting
	st SioRxCueBehind
	;Reception buffer reading point
	st SioRxCueRPnt
	;Reception buffer writing point
	st SioRxCueWPnt
	;Resets reception buffer overrun flag
	st SioOverRun
	;Receiving side transfer end interrupt enable
	set1 SCON1, 0
	;Receiving standby
	set1 SCON1, 3
	;SioInit end
	ret

;Serial communications end processing
SioEnd:
	;SCON0 = 0
	mov #0, SCON0
	;SCON1 = 0
	mov #0, SCON1
	;P1FCR = 0bfh
	mov #$bf, P1FCR
	;P1DDR = 0a4h
	mov #$a4, P1DDR
	;SioEnd end
	ret

;Sending 1 byte
;Inputs: 
;	acc: Transmission data
SioPut1:
	;Pushes the transmission data onto the stack
	push acc

splp1:
	;Waits until any previous transfer is completed
	ld SCON0
	bp acc, 3, splp1
	;Pops the transmission data off of the stack
	pop acc
	;Sets the data to be transferred
	st SBUF0
	;Starts sending
	set1 SCON0, 3
	;SioPut1 end
	ret

;Reading 1 Byte from the Reception Buffer (Asynchronous Reception)
;Outputs:
;	acc: 0 = Normal end
;	acc: 0ffh = No received data
;	acc: 0feh = Buffer overflow 
;	acc: 0fdh = Overrun error
;	b: Received data (Valid only in the case of normal end.)

;Waiting data amount check
SioGet1:
	;Waiting amount of data
	ld SioRxCueBehind
	;When waiting amount != 0
	bnz sgnx1
	;When waiting amount == 0
	mov #$ff, acc
	;Returns when acc = 0ffh (no received data)
	ret

;Buffer overflow detection
sgnx1:
	;SioRxCueBehind - SioRxCueSize
	;SioRxCueBehind == SioRxCueSize
	be #SioRxCueSize, sgnx3
	;SioRxCueBehind < SioRxCueSize
	bp PSW, 7, sgnx3
	;SioRxCueBehind > SioRxCueSize
	;When the buffer capacity has been exceeded
	mov #$fe, acc
	;Return when acc = 0feh (buffer overflow)
	ret

;Overrun error detection
sgnx3:
	;Overrun flag
	ld SioOverRun
	;Not detected
	bz sgnx4
	;Detected
	mov #$fd, acc
	;Return when acc = 0fdh (overrun error)
	ret

sgnx4:
	;dec waiting amount
	dec SioRxCueBehind
	;Calculating the received data reading point
	;r0 = SioRxCue + SioRxCueRPnt
	ld SioRxCueRPnt
	add #SioRxCue
	st 0
	;inc data reading point
	inc SioRxCueRPnt

	;If reading point = buffer size, then reading point is reset to 0
	ld SioRxCueRPnt
	;When SioRxCueRPnt != SioRxCueSize
	bne #SioRxCueSize, sgnx2
	;When SioRxCueRPnt == SioRxCueSize
	mov #0, SioRxCueRPnt

sgnx2:
	;Loads the input data into acc
	ld @r0
	;Stores the value in b
	st b
	;acc = acc = 0 (normal end, data exists)
	mov #0, acc
	;SioGet1 end
	ret

;Reading 1 byte from the reception buffer
;(If there is no received data, this routine waits until data is received)
;Outputs:
;	acc: 0 = Normal end
;	acc: 0feh = Buffer overflow
;	acc: 0fdh = Overrun error
;	b: Received data (Valid only in the case of normal end.)

SioGet1W:
	;Asynchronous reception
	call SioGet1
	;Waits until data is received
	be #$ff, SioGet1W
	;SioGet1W end
	ret

;Getting the Amount of Data Waiting in the Reception Buffer
;Output:
;	acc: Amount of data (bytes)

SioGetRxLen:
	;Amount waiting
	ld SioRxCueBehind
	;SioGetRxLen end
	ret

;SIO Reception Interrupt Handler
int_SioRx:
	;Pushes the register to be used onto the stack
	push acc
	push PSW
	;Selects data RAM bank 1
	set1 PSw, 1
	;Pushes the register onto the stack
	push 0

	;Calculating the writing point
	;r0 = SioRxCue + SioRxCueWPnt
	ld SioRxCueWPnt
	add #SioRxCue
	st 0
	;Loads the received data
	ld SBUF1
	;Writes the data to the buffer
	st @r0
	;Writing point ++
	inc SioRxCueWPnt

	;Resets the writing point once it reaches the buffer size
	ld SioRxCueWPnt
	bne #SioRxCueSize, isnx1
	mov #0, SioRxCueWPnt

isnx1:
	;Data waiting amount ++
	inc SioRxCueBehind
	;Resets the transfer end flag
	clr1 SCON1, 1
	;Checking the overrun error
	;If an overrun has not occurred, then isnx2
	bn SCON1, 6, isnx2
	;If an overrun has occurred -> Sets flag
	mov #1, SioOverRun
	;Resets overrun flag
	clr1 SCON1, 6

isnx2:
	;Starts the next transfer
	set1 SCON1, 3
	;Pops the register to be used off of the stack
	pop 0
	pop PSW
	pop acc
	;int_SioRx end
	reti

;Displaying a two digit value
;Inputs
;	acc: Numeric value
;	c: Horizontal position of character
;	b: Vertical position of character
put2digit:
	;Pushes the coordinate data onto the stack
	push b
	push c
	;Calculates the tens digit and the ones digit
	st c
	;( acc = acc/10, work1 = acc mod 10 )
	xor acc
	mov #10, b
	div
	ld b
	;Stores the ones digit in work1
	st work1
	ld c
	;Pops the coordinate values into (c, b)
	pop c
	pop b
	;Pushes the coordinates onto the stack again
	push b
	push c
	;Displays the tens digit
	call putch
	;Loads the ones digit
	ld work1
	;Pops the coordinate values into (c, b)
	pop c
	pop b
	;Moves the display coordinates to the right
	inc c
	;Displays the ones digit
	call putch
	;Put2digit end
	ret

;Clears the LCD display image
cls:
	;Pushes the OCR value onto the stack
	push OCR
	;Specifies the system clock
	mov #osc_rc, OCR
	;Specifies the display RAM bank address (BANK0)
	mov #0, XBNK
	;Clears the data in that bank
	call cls_s
	;Specifies the display RAM bank address (BANK1)
	mov #1, XBNK
	;Clears the data in that bank
	call cls_s
	;Pops the OCR value onto the stack
	pop OCR
	;Cls end
	ret

;Clearing one bank of display RAM
cls_s:
	;Points the indirect addressing register at the start of display RAM
	mov #$80, 2
	;Sets the number of loops in loop counter b
	mov #$80, b
loop3:
	;Writes "0" while incrementing the address
	mov #0, @r2
	inc 2
	;Repeats until b is "0"
	dbnz b, loop3
	;Cls_s end
	ret

;Displaying one character at a specified position
;Inputs
;	acc: Character code
;	c: Horizontal position of character
;	b: Vertical position of character
putch:
	push XBNK
	push acc
	;Calculates display RAM address according to coordinates
	call locate
	pop acc
	;Displays one character
	call put_chara
	pop XBNK
	;Putch end
	ret

;Calculating the display RAM address according to the display position specification
;Inputs
;	c: Horizontal position (0 to 5)
;	b: Vertical position (0 to 3)
;Outputs
;	2: RAM address
;	XBNK: display RAM bank
;Determining the display RAM bank address
locate:
	;Jump to next1 when b >= 2
	ld b
	sub #2
	bn PSW, 7, next1
	;Specifies the display RAM bank address (BANK0)
	mov #$0, XBNK
	br next2

next1:
	st b
	;Specifies the display RAM bank address (BANK1)
	mov #$01, XBNK

;Calculating the RAM address for a specified position on the display
next2:
	;b * 40h + c + 80h
	ld b
	rol
	rol
	rol
	rol
	rol
	rol
	add c
	add #$80
	;Stores the RAM address in 2
	st 2
	;Locate end
	ret

put_chara:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 1
	set1 PSW, 1
	;Calculating the character data address
	;(TRH,TRL) = acc*8 + fontdata
	rol
	rol
	rol
	add #<fontdata
	st TRL
	mov #0, acc
	addc #>fontdata
	st TRH
	;Pushes the OCR value onto the stack
	push OCR
	;Specifies the system clock
	mov #osc_rc, OCR
	;Offset value for loading the character data
	mov #0, b
	;Loop counter
	mov #4, c
loop1:
	;Loads the display data for the first line
	ld b
	ldc
	;Increments the load data offset by 1
	inc b
	;Transfers the display data to display RAM
	st @r2
	ld 2
	;Adds 6 to the display RAM address
	add #6
	st 2

	;Loads the display data for the second line
	ld b
	ldc
	;Increments the load data offset by 1
	inc b
	;Transfers the display data to display RAM
	st @r2
	ld 2
	;Adds 10 to the display RAM address
	add #10
	st 2

	;Decrements the loop counter
	dec c
	ld c
	;Repeats for 8 lines (four times)
	bnz loop1
	;Pops the OCR value off of the stack
	pop OCR
	;Pops the PSW value off of the stack
	pop PSW
	;Put_chara end
	ret

;Character bit image data
fontdata:
	.byte $7c, $e6, $c6, $c6, $c6, $ce, $7c, $00	; 0
	.byte $18, $38, $18, $18, $18, $18, $3c, $00	; 1
	.byte $7c, $c6, $c6, $0c, $38, $60, $fe, $00	; 2
	.byte $7c, $e6, $06, $1c, $06, $e6, $7c, $00	; 3
	.byte $0c, $1c, $3c, $6c, $cc, $fe, $0c, $00	; 4
	.byte $fe, $c0, $fc, $06, $06, $c6, $7c, $00	; 5
	.byte $1c, $30, $60, $fc, $c6, $c6, $7c, $00	; 6
	.byte $fe, $c6, $04, $0c, $18, $18, $38, $00	; 7
	.byte $7c, $c6, $c6, $7c, $c6, $c6, $7c, $00	; 8
	.byte $7c, $c6, $c6, $7e, $06, $0c, $78, $00	; 9

;Low battery automatic detection function ON
BattChkOn:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 0
	clr1 PSW, 1
	;Detects low battery (0)
	mov #0, LowBattChk
	;Pops the PSW value off of the stack
	pop PSW
	;BattChkOn end
	ret

;Low battery automatic detection function OFF
BattChkOff:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 0
	clr1 PSW, 1
	;Does not detect low battery (0ffh)
	mov #$ff, LowBattChk
	;Pops the PSW value off of the stack
	pop PSW
	;BattChkOff end
	ret

;Base timer interrupt handler
int_BaseTimer:
	;Clears the base timer interrupt source
	clr1 BTCR, 1
	;Counter ++
	inc bcount
	;User interrupt processing end
	ret

* ------------------------------------------------------
*  End Program

	.include "libraries/endprog.i"

